#include "m4Warmer.h"
#include "warmerDriver.h"
#include "warmerClient.h"

MarkIV_Warmer::MarkIV_Warmer(WarmerDriver& theDriver, WarmerClient& theClient)
: Warmer(theDriver, theClient)
, itsState(notBrewing)
{GetDriver().WarmerOff();}

MarkIV_Warmer::~MarkIV_Warmer() {}

void MarkIV_Warmer::Brew()
{
    switch (itsState)
    {
    case notBrewing:       itsState=brewingPotEmpty; break;
    case brewingPotEmpty:  /* ignore */ break;
    case brewingWarming:   /* ignore */ break;
    case emptyPotRemoved:  /* ignore */ break;
    case fullPotRemoved:   /* ignore */ break;
    case brewingComplete:  /* ignore */ break;
    case brewedPotRemoved: /* ignore */ break;
    }
}

void MarkIV_Warmer::BrewingComplete()
{
    switch (itsState)
    {
    case notBrewing:       /* ignore */ break;
    case brewingPotEmpty:  itsState=notBrewing;
	                   GetClient().CycleComplete(); break;
    case brewingWarming:   itsState=brewingComplete; break;
    case emptyPotRemoved:  itsState=notBrewing;
			   GetClient().CycleComplete(); break;
    case fullPotRemoved:   itsState=brewedPotRemoved; break;
    case brewingComplete:  /* ignore */ break;
    case brewedPotRemoved: /* ignore */ break;
    }
}

void MarkIV_Warmer::PotEmpty()
{
    switch (itsState)
    {
    case notBrewing:       /* ignore */ break;
    case brewingPotEmpty:  /* ignore */ break;
    case brewingWarming:   itsState=brewingPotEmpty;
	                   GetDriver().WarmerOff(); break;
    case emptyPotRemoved:
    case fullPotRemoved :  itsState=brewingPotEmpty;
			   GetClient().Pot(); break;
    case brewingComplete:  itsState=notBrewing;
			   GetDriver().WarmerOff();
			   GetClient().CycleComplete(); break;
    case brewedPotRemoved: itsState=notBrewing;
			   GetClient().CycleComplete(); break;
    }
}

void MarkIV_Warmer::PotNotEmpty()
{
    switch (itsState)
    {
    case notBrewing:       /* ignore */ break;
    case brewingPotEmpty:  itsState=brewingWarming;
	                   GetDriver().WarmerOn(); break;
    case brewingWarming:   /* ignore */ break;
    case emptyPotRemoved:  
    case fullPotRemoved:   itsState=brewingWarming;
			   GetDriver().WarmerOn();
			   GetClient().Pot(); break;
    case brewingComplete:  /* ignore */ break;
    case brewedPotRemoved: itsState=brewingComplete;
			   GetDriver().WarmerOn(); break;
    }
}

void MarkIV_Warmer::WarmerEmpty()
{
    switch (itsState)
    {
    case notBrewing:       /* ignore */ break;
    case brewingPotEmpty:  itsState=emptyPotRemoved;
	                   GetClient().NoPot(); break;
    case brewingWarming:   itsState=fullPotRemoved;
			   GetClient().NoPot();
			   GetDriver().WarmerOff(); break;
    case emptyPotRemoved:  /* ignore */ break;
    case fullPotRemoved:   /* ignore */ break;
    case brewingComplete:  itsState=brewedPotRemoved;
			   GetDriver().WarmerOff(); break;
    case brewedPotRemoved: /* ignore */ break;
    }
}

void MarkIV_Warmer::Poll()
{
    switch (GetDriver().GetWarmerStatus())
    {
    case WarmerDriver::noPot:       WarmerEmpty(); break;
    case WarmerDriver::potEmpty:    PotEmpty();    break;
    case WarmerDriver::potNotEmpty: PotNotEmpty(); break;
    }
}

int MarkIV_Warmer::CheckReady()
{
    return GetDriver().GetWarmerStatus() == WarmerDriver::potEmpty;
}
