#include <stdio.h>
#include <windows.h>
#include <comdef.h>

// this loads the visualisation library
#import "../VisualPart.vb/Mark4.exe"
Mark4::_Mark4VisualisationPtr pM4Visualisation;

// book version
#include "Mark4 Source/coffeeMaker.h"
#include "Mark4 Source/m4Sprayer.h"
#include "Mark4 Source/m4Warmer.h"
#include "Mark4 Source/markIV_UI.h"
#include "Mark4 Source/m4UIDriver.h"
#include "Mark4 Source/m4SpDriver.h"
#include "Mark4 Source/m4WmDriver.h"


// realisation of export functions
// in reality these methods need not to be implemented
// also the visualisation can be done by the coffee maker itself :-)
WarmerPlateStatus GetWarmerPlateStatus()
{
    return( ( WarmerPlateStatus )pM4Visualisation->GetWarmerPlateStatus() );
}

BoilerStatus GetBoilerStatus()
{
    return( ( BoilerStatus )pM4Visualisation->GetBoilerStatus() );
}

BrewButtonStatus GetBrewButtonStatus()
{
    return( ( BrewButtonStatus )pM4Visualisation->GetBrewButtonStatus() );
}

void SetBoilerState(BoilerHeaterState s)
{
    long uS = (long)s;
    pM4Visualisation->SetBoilerState( &uS );
}

void SetWarmerState(WarmerState s)
{
    long uS = (long)s;
    pM4Visualisation->SetWarmerState( &uS );
}

void SetIndicatorState(IndicatorState s)
{
    long uS = (long)s;
    pM4Visualisation->SetIndicatorState( &uS );
}

void SetReliefValveState(ReliefValveState s)
{
    long uS = (long)s;
    pM4Visualisation->SetReliefValveState( &uS );
}



// here to this thread is passed the id of the main thread
// it will send the WM_QUIT message when the visualisation com object tells that the
// visualisation window was closed
DWORD WINAPI AutomateThread( void* pId )
{
    DWORD* pMainThreadId = (DWORD*) pId;

    OleInitialize( 0 );

    // here we create com object which later will create visualisation window
    // this approach was selected because it is much easier to create visualisation window
    // in visual basic then to do it in visual c++
    HRESULT hErr = pM4Visualisation.CreateInstance( "Mark4.Mark4Visualisation" );//__uuidof( Mark4::Mark4Visualisation ) );
    if ( ERROR_SUCCESS != hErr )
    {
        printf( "Error %i: create instance", hErr );
        PostThreadMessage( *pMainThreadId, WM_QUIT, 1, 0 );
        return( 1 );
    }

    CoffeeMaker cm;
    MarkIV_WarmerDriver     warmerDriver;
    MarkIV_SprayerDriver    sprayerDriver;
    MarkIV_UIDriver         uiDriver;
    MarkIV_Sprayer          sprayer(sprayerDriver, cm);
    MarkIV_Warmer           warmer(warmerDriver, cm);
    MarkIV_UI               ui(uiDriver, cm);
    cm.SetUI(ui);
    cm.SetSprayer(sprayer);
    cm.SetWarmer(warmer);


    
    while ( true )
    {
        if ( false == pM4Visualisation->FormAlive() )
        {
            // if the visualisation window was closed we will close this thread
            // and send the quit message to the main thread
            pM4Visualisation.Release();
            CoUninitialize();

            PostThreadMessage( *pMainThreadId, WM_QUIT, 0, 0 );
            return( 0 );
        }

        
        try
        {

            ui.Poll();
            sprayer.Poll();
            warmer.Poll();

        }
        catch( _com_error cErr )
        {
            pM4Visualisation.Release();
            CoUninitialize();

            PostThreadMessage( *pMainThreadId, WM_QUIT, 1, 0 );
            return( 1 );
        }
        
        // we tell the thread to sleep for 0.1 second
        Sleep( 20 );    
    }
}

int main(int argc, char* argv[])
{
    DWORD   uMainThreadId       = GetCurrentThreadId();
    DWORD   uAutomateThreadId   = 0;
    HANDLE  hAutomateThread = CreateThread( 0, 0, &AutomateThread, &uMainThreadId, 0, &uAutomateThreadId );

    MSG     sMessage;
    BOOL    bResult;
    while( 0 != (bResult = GetMessage( &sMessage, NULL, 0, 0 )) )
    { 
        if ( -1 == bResult )
        {
            // some error occured
            __asm int 3

            return( 1 );
        }
        else
        {
            TranslateMessage( &sMessage );
            DispatchMessage( &sMessage );
        }
    }

    return( sMessage.wParam );
}